////////////////////////////////////////////////////////////////////////////////
//                                                                            //
//   Wireless Communication Library 7                                         //
//                                                                            //
//   Copyright (C) 2006-2024 Mike Petrichenko                                 //
//                           Soft Service Company                             //
//                           All Rights Reserved                              //
//                                                                            //
//   http://www.btframework.com                                               //
//                                                                            //
//   support@btframework.com                                                  //
//   shop@btframework.com                                                     //
//                                                                            //
////////////////////////////////////////////////////////////////////////////////

#pragma once

#include "..\Common\wclErrors.h"

using namespace wclCommon;

namespace wclTimeline
{
	/* Common Timeline error codes. */

	/// <summary> The base error code for all Windows Timeline Framework errors. </summary>
	const int WCL_E_TIMELINE_BASE = 0x00070000;

	/* Timeline Channel error codes. */

	/// <summary> The base error code for Timeline Channel errors. </summary>
	const int WCL_E_TIMELINE_CHANNEL_BASE = WCL_E_TIMELINE_BASE + 0x0000;
	/// <summary> The Windows Timeline features are supported on Windows 10 16299
	///   and above only. </summary>
	const int WCL_E_TIMELINE_CHANNEL_FEATURE_NOT_SUPPORTED = WCL_E_TIMELINE_CHANNEL_BASE + 0x0000;
	/// <summary> The Windows Timeline Channel has not been opened. </summary>
	const int WCL_E_TIMELINE_CHANNEL_NOT_ACTIVE = WCL_E_TIMELINE_CHANNEL_BASE + 0x0001;
	/// <summary> The Windows Timeline Channel has already been opened. </summary>
	const int WCL_E_TIMELINE_CHANNEL_ACTIVE = WCL_E_TIMELINE_CHANNEL_BASE + 0x0002;
	/// <summary> Unable to unregister application activation protocol. </summary>
	const int WCL_E_TIMELINE_CHANNEL_UNREGISTER_PROTOCOL_FAILED = WCL_E_TIMELINE_CHANNEL_BASE + 0x0003;
	/// <summary> Failed to create WinRT thread termination event. </summary>
	const int WCL_E_TIMELINE_CHANNEL_CREATE_TERM_EVENT_FAILED = WCL_E_TIMELINE_CHANNEL_BASE + 0x0004;
	/// <summary> Failed to create WinRT thread initialization event. </summary>
	const int WCL_E_TIMELINE_CHANNEL_CREATE_INIT_EVENT_FAILED = WCL_E_TIMELINE_CHANNEL_BASE + 0x0005;
	/// <summary> Unable to create WinRT thread. </summary>
	const int WCL_E_TIMELINE_CHANNEL_CREATE_WINRT_THREAD_FAILED = WCL_E_TIMELINE_CHANNEL_BASE + 0x0006;
	/// <summary> Failed to get the default Windows Timeline channel. </summary>
	const int WCL_E_TIMELINE_CHANNEL_GET_DEFAULT_FAILED = WCL_E_TIMELINE_CHANNEL_BASE + 0x0007;
	/// <summary> Unable to create registry key for an Application Activation Protocol. </summary>
	const int WCL_E_TIMELINE_CHANNEL_CREATE_PROTOCOL_KEY_FAILED = WCL_E_TIMELINE_CHANNEL_BASE + 0x0008;
	/// <summary> Failed to write application protocol display name. </summary>
	const int WCL_E_TIMELINE_CHANNEL_SET_PROTOCOL_NAME_FAILED = WCL_E_TIMELINE_CHANNEL_BASE + 0x0009;
	/// <summary> Unable to set properties for the Application Activation Protocol
	///   registry entry. </summary>
	const int WCL_E_TIMELINE_CHANNEL_SET_PROTOCOL_PROPERTY_FAILED = WCL_E_TIMELINE_CHANNEL_BASE + 0x000A;
	/// <summary> Unable to create Application Activation command registry entry. </summary>
	const int WCL_E_TIMELINE_CHANNEL_CREATE_COMMAND_KEY_FAILED = WCL_E_TIMELINE_CHANNEL_BASE + 0x000B;
	/// <summary> Unable to get application executable name and path. </summary>
	const int WCL_E_TIMELINE_CHANNEL_GET_APP_NAME_FAILED = WCL_E_TIMELINE_CHANNEL_BASE + 0x000C;
	/// <summary> Unable to set application startup command for the application
	///   activation protocol. </summary>
	const int WCL_E_TIMELINE_CHANNEL_SET_RUN_CMD_FAILED = WCL_E_TIMELINE_CHANNEL_BASE + 0x000D;
	/// <summary> Unable to create or open User Activity. </summary>
	const int WCL_E_TIMELINE_CHANNEL_CREATE_ACTIVITY_FAILED = WCL_E_TIMELINE_CHANNEL_BASE + 0x000E;
	/// <summary> Unable to get waitable interface for the asynchronous operation. </summary>
	const int WCL_E_TIMELINE_CHANNEL_GET_ASYNC_INFO_FAILED = WCL_E_TIMELINE_CHANNEL_BASE + 0x000F;
	/// <summary> Unable to get UserActivity creation result. </summary>
	const int WCL_E_TIMELINE_CHANNEL_GET_ASYNC_RESULT_FAILED = WCL_E_TIMELINE_CHANNEL_BASE + 0x0010;
	/// <summary> User Activity has not been created or opened. Interface not found. </summary>
	const int WCL_E_TIMELINE_CHANNEL_ACTIVITY_NOT_CREATED = WCL_E_TIMELINE_CHANNEL_BASE + 0x0011;

	/* Timeline User Activity error codes. */

	/// <summary> The base error code for User Activity object. </summary>
	const int WCL_E_TIMELINE_USER_ACTIVITY_BASE = WCL_E_TIMELINE_BASE + 0x1000;
	/// <summary> The User Activity has already been initialized. </summary>
	const int WCL_E_TIMELINE_USER_ACTIVITY_INITIALIZED = WCL_E_TIMELINE_USER_ACTIVITY_BASE + 0x0000;
	/// <summary> Failed to get User Activity ID. </summary>
	const int WCL_E_TIMELINE_USER_ACTIVITY_GET_ID_FAILED = WCL_E_TIMELINE_USER_ACTIVITY_BASE + 0x0001;
	/// <summary> Unable to get User Activity state. </summary>
	const int WCL_E_TIMELINE_USER_ACTIVITY_GET_STATE_FAILED = WCL_E_TIMELINE_USER_ACTIVITY_BASE + 0x0002;
	/// <summary> Unable to get Activity's Visual Elements interface. </summary>
	const int WCL_E_TIMELINE_USER_ACTIVITY_GET_VISUAL_ELEMENTS_FAILED = WCL_E_TIMELINE_USER_ACTIVITY_BASE + 0x0003;
	/// <summary> Unable to get User Activity Description. </summary>
	const int WCL_E_TIMELINE_USER_ACTIVITY_GET_DESCRIPTION_FAILED = WCL_E_TIMELINE_USER_ACTIVITY_BASE + 0x0004;
	/// <summary> Unable to get User Activity display text. </summary>
	const int WCL_E_TIMELINE_USER_ACTIVITY_GET_DISPLAY_TEXT_FAILED = WCL_E_TIMELINE_USER_ACTIVITY_BASE + 0x0005;
	/// <summary> Unable to get URI management class. </summary>
	const int WCL_E_TIMELINE_USER_ACTIVITY_GET_URI_CLASS_FAILED = WCL_E_TIMELINE_USER_ACTIVITY_BASE + 0x0006;
	/// <summary> Unable to get URI. </summary>
	const int WCL_E_TIMELINE_USER_ACTIVITY_GET_URI_FAILED = WCL_E_TIMELINE_USER_ACTIVITY_BASE + 0x0007;
	/// <summary> The User Activity has not been initialized. </summary>
	const int WCL_E_TIMELINE_USER_ACTIVITY_NOT_INITIALIZED = WCL_E_TIMELINE_USER_ACTIVITY_BASE + 0x0008;
	/// <summary> User Activity session has been started. </summary>
	const int WCL_E_TIMELINE_USER_ACTIVITY_SESSION_ACTIVE = WCL_E_TIMELINE_USER_ACTIVITY_BASE + 0x0009;
	/// <summary> Other User Activity object has already started session. </summary>
	const int WCL_E_TIMELINE_USER_ACTIVITY_SESSION_RUNNING = WCL_E_TIMELINE_USER_ACTIVITY_BASE + 0x000A;
	/// <summary> Unable to set User Activity description. </summary>
	const int WCL_E_TIMELINE_USER_ACTIVITY_SET_DESCRIPTION_FAILED = WCL_E_TIMELINE_USER_ACTIVITY_BASE + 0x000B;
	/// <summary> Unable to set User Activity display text. </summary>
	const int WCL_E_TIMELINE_USER_ACTIVITY_SET_DISPLAY_TEXT_FAILED = WCL_E_TIMELINE_USER_ACTIVITY_BASE + 0x000C;
	/// <summary> Unable to create URI management class. </summary>
	const int WCL_E_TIMELINE_USER_ACTIVITY_CREATE_URI_CLASS_FAILED = WCL_E_TIMELINE_USER_ACTIVITY_BASE + 0x000D;
	/// <summary> Unable to set User Activity activation URI. </summary>
	const int WCL_E_TIMELINE_USER_ACTIVITY_SET_ACTIVATION_URI_FAILED = WCL_E_TIMELINE_USER_ACTIVITY_BASE + 0x000E;
	/// <summary> Unable to save User Activity. </summary>
	const int WCL_E_TIMELINE_USER_ACTIVITY_SAVE_FAILED = WCL_E_TIMELINE_USER_ACTIVITY_BASE + 0x000F;
	/// <summary> Unable to get status of the User Activity save operation. </summary>
	const int WCL_E_TIMELINE_USER_ACTIVITY_GET_SAVE_STATUS_FAILED = WCL_E_TIMELINE_USER_ACTIVITY_BASE + 0x0010;
	/// <summary> Unable to start User Activity session. </summary>
	const int WCL_E_TIMELINE_USER_ACTIVITY_CREATE_SESSION_FAILED = WCL_E_TIMELINE_USER_ACTIVITY_BASE + 0x0011;
	/// <summary> The USer Activity session has not been started. </summary>
	const int WCL_E_TIMELINE_USER_ACTIVITY_SESSION_NOT_STARTED = WCL_E_TIMELINE_USER_ACTIVITY_BASE + 0x0012;
	/// <summary> Unable to get Activity's Visual Elements extended interface. </summary>
	const int WCL_E_TIMELINE_USER_ACTIVITY_GET_VISUAL_ELEMENTS2_FAILED = WCL_E_TIMELINE_USER_ACTIVITY_BASE + 0x0013;
	/// <summary> Unable to get Activity's title caption. </summary>
	const int WCL_E_TIMELINE_USER_ACTIVITY_GET_TITLE_CAPTION_FAILED = WCL_E_TIMELINE_USER_ACTIVITY_BASE + 0x0014;
	/// <summary> Unable to set Activity's title caption. </summary>
	const int WCL_E_TIMELINE_USER_ACTIVITY_SET_TITLE_CAPTION_FAILED = WCL_E_TIMELINE_USER_ACTIVITY_BASE + 0x0015;

	/// <summary> The base exception class for Timeline Framework exceptions. </summary>
	/// <seealso cref="wclException" />
	class wclETimeline : public wclException
	{
	public:
		/// <summary> Creates new exception object. </summary>
		/// <param name="msg"> The exception message. </param>
		wclETimeline(const char* msg) : wclException(msg) { };
	};

	/// <summary> The base class for Timeline Channel exceptions. </summary>
	/// <seealso cref="wclETimeline" />
	class wclETimelineChannel : public wclETimeline
	{
	public:
		/// <summary> Creates new exception object. </summary>
		/// <param name="msg"> The exception message. </param>
		wclETimelineChannel(const char* msg) : wclETimeline(msg) { };
	};

	/// <summary> The base class for Timeline User Activity exceptions. </summary>
	/// <seealso cref="wclETimeline" />
	class wclEUserActivity : public wclETimeline
	{
	public:
		/// <summary> Creates new exception object. </summary>
		/// <param name="msg"> The exception message. </param>
		wclEUserActivity(const char* msg) : wclETimeline(msg) { };
	};

	/// <summary> The exception raises when an application changes User Activity
	///   properties when an User Activity session is active. </summary>
	/// <seealso cref="wclEUserActivity" />
	class wclEUserActivitySessionActive : public wclEUserActivity
	{
	public:
		/// <summary> Creates new exception object. </summary>
		/// <param name="msg"> The exception message. </param>
		wclEUserActivitySessionActive(const char* msg) : wclEUserActivity(msg) { };
	};

	/// <summary> The exception raises when an application changes properties of
	///   the not initialized User Activity object. </summary>
	/// <seealso cref="wclEUserActivity" />
	class wclEUserActivityNotInitialized : public wclEUserActivity
	{
	public:
		/// <summary> Creates new exception object. </summary>
		/// <param name="msg"> The exception message. </param>
		wclEUserActivityNotInitialized(const char* msg) : wclEUserActivity(msg) { };
	};
}