////////////////////////////////////////////////////////////////////////////////
//                                                                            //
//   Wireless Communication Library 7                                         //
//                                                                            //
//   Copyright (C) 2006-2024 Mike Petrichenko                                 //
//                           Soft Service Company                             //
//                           All Rights Reserved                              //
//                                                                            //
//   http://www.btframework.com                                               //
//                                                                            //
//   support@btframework.com                                                  //
//   shop@btframework.com                                                     //
//                                                                            //
////////////////////////////////////////////////////////////////////////////////

#pragma once

#include "..\Common\wclWinAPI.h"

namespace wclTimeline
{
	namespace UaApi
	{
		// WinRT User Activity API.

		#define UserActivityChannelName (_T("Windows.ApplicationModel.UserActivities.UserActivityChannel"))

		/* User Activity API enumerations. */

		typedef enum
		{
			UserActivityState_New = 0,
			UserActivityState_Published = 1
		} UserActivityState;

		/* Forward declarations. */

		typedef interface IUserActivityVector IUserActivityVector;
		typedef interface IUserActivitySessionHistoryItemVector IUserActivitySessionHistoryItemVector;
		typedef interface IUserActivityVectorView IUserActivityVectorView;
		typedef interface IUserActivitySessionHistoryItemVectorView IUserActivitySessionHistoryItemVectorView;
		typedef interface IUserActivityIterable IUserActivityIterable;
		typedef interface IUserActivityIterator IUserActivityIterator;
		typedef interface IUserActivity IUserActivity;
		typedef interface IUserActivity2 IUserActivity2;
		typedef interface IUserActivity3 IUserActivity3;
		typedef interface IUserActivityAttribution IUserActivityAttribution;
		typedef interface IUserActivityAttributionFactory IUserActivityAttributionFactory;
		typedef interface IUserActivityChannel IUserActivityChannel;
		typedef interface IUserActivityChannel2 IUserActivityChannel2;
		typedef interface IUserActivityContentInfo IUserActivityContentInfo;
		typedef interface IUserActivityFactory IUserActivityFactory;
		typedef interface IUserActivityRequest IUserActivityRequest;
		typedef interface IUserActivityRequestManager IUserActivityRequestManager;
		typedef interface IUserActivitySession IUserActivitySession;
		typedef interface IUserActivitySessionHistoryItem IUserActivitySessionHistoryItem;
		typedef interface IUserActivityVisualElements IUserActivityVisualElements;
		typedef interface IUserActivityVisualElements2 IUserActivityVisualElements2;
		typedef interface IUserActivityChannelStatics IUserActivityChannelStatics;
		typedef interface IUserActivityChannelStatics2 IUserActivityChannelStatics2;
		typedef interface IUserActivityContentInfoStatics IUserActivityContentInfoStatics;
		typedef interface IUserActivityRequestManagerStatics IUserActivityRequestManagerStatics;
		typedef interface IUserActivityStatics IUserActivityStatics;
		typedef interface IUserActivityAsyncOperation IUserActivityAsyncOperation;
		typedef interface IUserActivitySessionHistoryItemAsyncOperation IUserActivitySessionHistoryItemAsyncOperation;
		typedef interface IUserActivityRequestedEvent IUserActivityRequestedEvent;
		typedef interface IUserActivityAsyncOperationCompletedHandler IUserActivityAsyncOperationCompletedHandler;
		typedef interface IUserActivitySessionHistoryItemAsyncOperationCompletedHandler IUserActivitySessionHistoryItemAsyncOperationCompletedHandler;
		typedef interface IUserActivityRequestedEventArgs IUserActivityRequestedEventArgs;

		/* Interface declarations. */

		MIDL_INTERFACE("e3004e95-0b05-55dc-bf3b-be06fae03152")
		IUserActivityVector : public wclCommon::WinApi::IInspectable
		{
		public:
			virtual HRESULT STDMETHODCALLTYPE GetAt(DWORD index, IUserActivity** item) = 0;
			virtual HRESULT STDMETHODCALLTYPE get_Size(DWORD* size) = 0;
			virtual HRESULT STDMETHODCALLTYPE GetView(IUserActivityVectorView** view) = 0;
			virtual HRESULT STDMETHODCALLTYPE IndexOf(IUserActivity* item, DWORD* index,
				BOOLEAN* found) = 0;
			virtual HRESULT STDMETHODCALLTYPE SetAt(DWORD index, IUserActivity* item) = 0;
			virtual HRESULT STDMETHODCALLTYPE InsertAt(DWORD index, IUserActivity* item) = 0;
			virtual HRESULT STDMETHODCALLTYPE RemoveAt(DWORD index) = 0;
			virtual HRESULT STDMETHODCALLTYPE Append(IUserActivity* item) = 0;
			virtual HRESULT STDMETHODCALLTYPE RemoveAtEnd() = 0;
			virtual HRESULT STDMETHODCALLTYPE Clear() = 0;
			virtual HRESULT STDMETHODCALLTYPE GetMany(DWORD startIndex, DWORD capacity,
				IUserActivity** items, DWORD* actual) = 0;
			virtual HRESULT STDMETHODCALLTYPE ReplaceAll(DWORD count,
				IUserActivity** value) = 0;
		};

		MIDL_INTERFACE("8765461c-2f90-586e-83ec-58b3e4309480")
		IUserActivitySessionHistoryItemVector : public wclCommon::WinApi::IInspectable
		{
		public:
			virtual HRESULT STDMETHODCALLTYPE GetAt(DWORD index, IUserActivitySessionHistoryItem** item) = 0;
			virtual HRESULT STDMETHODCALLTYPE get_Size(DWORD* size) = 0;
			virtual HRESULT STDMETHODCALLTYPE GetView(IUserActivitySessionHistoryItemVectorView** view) = 0;
			virtual HRESULT STDMETHODCALLTYPE IndexOf(IUserActivitySessionHistoryItem* item,
				DWORD index, BOOLEAN* found) = 0;
			virtual HRESULT STDMETHODCALLTYPE SetAt(DWORD index, IUserActivitySessionHistoryItem* item) = 0;
			virtual HRESULT STDMETHODCALLTYPE InsertAt(DWORD index, IUserActivitySessionHistoryItem* item) = 0;
			virtual HRESULT STDMETHODCALLTYPE RemoveAt(DWORD index) = 0;
			virtual HRESULT STDMETHODCALLTYPE Append(IUserActivitySessionHistoryItem* item) = 0;
			virtual HRESULT STDMETHODCALLTYPE RemoveAtEnd() = 0;
			virtual HRESULT STDMETHODCALLTYPE Clear() = 0;
			virtual HRESULT STDMETHODCALLTYPE GetMany(DWORD startIndex, DWORD capacity,
				IUserActivitySessionHistoryItem** items, DWORD* actual) = 0;
			virtual HRESULT STDMETHODCALLTYPE ReplaceAll(DWORD count, IUserActivitySessionHistoryItem** value) = 0;
		};

		MIDL_INTERFACE("192a5116-61d6-5e18-8679-0af4f7090816")
		IUserActivityVectorView : public wclCommon::WinApi::IInspectable
		{
		public:
			virtual HRESULT STDMETHODCALLTYPE GetAt(DWORD index, IUserActivity** item) = 0;
			virtual HRESULT STDMETHODCALLTYPE get_Size(DWORD* size) = 0;
			virtual HRESULT STDMETHODCALLTYPE IndexOf(IUserActivity* item, DWORD* index, BOOLEAN* found) = 0;
			virtual HRESULT STDMETHODCALLTYPE GetMany(DWORD startIndex, DWORD capacity, IUserActivity** items,
				DWORD* actual) = 0;
		};

		MIDL_INTERFACE("006a83c1-59ff-5870-8d8d-0583814af160")
		IUserActivitySessionHistoryItemVectorView : public wclCommon::WinApi::IInspectable
		{
		public:
			virtual HRESULT STDMETHODCALLTYPE GetAt(DWORD index, IUserActivitySessionHistoryItem** item) = 0;
			virtual HRESULT STDMETHODCALLTYPE get_Size(DWORD* size) = 0;
			virtual HRESULT STDMETHODCALLTYPE IndexOf(IUserActivitySessionHistoryItem* item, DWORD* index,
				BOOLEAN* found) = 0;
			virtual HRESULT STDMETHODCALLTYPE GetMany(DWORD startIndex, DWORD capacity,
				IUserActivitySessionHistoryItem** items, DWORD* actual) = 0;
		};

		MIDL_INTERFACE("37e6ab55-1f30-5622-9778-1bdc07ac799f")
		IUserActivityIterable : public wclCommon::WinApi::IInspectable
		{
		public:
			virtual HRESULT STDMETHODCALLTYPE First(IUserActivityIterator** first) = 0;
		};

		MIDL_INTERFACE("2ca0633b-0505-5f88-a98f-8e7c5b08f25b")
		IUserActivityIterator : public wclCommon::WinApi::IInspectable
		{
		public:
			virtual HRESULT STDMETHODCALLTYPE get_Current(IUserActivity** current) = 0;
			virtual HRESULT STDMETHODCALLTYPE get_HasCurrent(BOOLEAN* hasCurrent) = 0;
			virtual HRESULT STDMETHODCALLTYPE MoveNext(BOOLEAN* hasCurrent) = 0;
			virtual HRESULT STDMETHODCALLTYPE GetMany(DWORD capacity, IUserActivity** items,
				DWORD* actual) = 0;
		};

		MIDL_INTERFACE("FC103E9E-2CAB-4D36-AEA2-B4BB556CEF0F")
		IUserActivity : public wclCommon::WinApi::IInspectable
		{
		public:
			virtual HRESULT STDMETHODCALLTYPE get_State(UserActivityState* value) = 0;
			virtual HRESULT STDMETHODCALLTYPE get_ActivityId(wclCommon::WinApi::HSTRING* value) = 0;
			virtual HRESULT STDMETHODCALLTYPE get_VisualElements(IUserActivityVisualElements** value) = 0;
			virtual HRESULT STDMETHODCALLTYPE get_ContentUri(wclCommon::WinApi::IUriRuntimeClass** value) = 0;
			virtual HRESULT STDMETHODCALLTYPE put_ContentUri(wclCommon::WinApi::IUriRuntimeClass* value) = 0;
			virtual HRESULT STDMETHODCALLTYPE get_ContentType(wclCommon::WinApi::HSTRING* value) = 0;
			virtual HRESULT STDMETHODCALLTYPE put_ContentType(wclCommon::WinApi::HSTRING value) = 0;
			virtual HRESULT STDMETHODCALLTYPE get_FallbackUri(wclCommon::WinApi::IUriRuntimeClass** value) = 0;
			virtual HRESULT STDMETHODCALLTYPE put_FallbackUri(wclCommon::WinApi::IUriRuntimeClass* value) = 0;
			virtual HRESULT STDMETHODCALLTYPE get_ActivationUri(wclCommon::WinApi::IUriRuntimeClass** value) = 0;
			virtual HRESULT STDMETHODCALLTYPE put_ActivationUri(wclCommon::WinApi::IUriRuntimeClass* value) = 0;
			virtual HRESULT STDMETHODCALLTYPE get_ContentInfo(IUserActivityContentInfo** value) = 0;
			virtual HRESULT STDMETHODCALLTYPE put_ContentInfo(IUserActivityContentInfo* value) = 0;
			virtual HRESULT STDMETHODCALLTYPE SaveAsync(wclCommon::WinApi::IAsyncAction** operation) = 0;
			virtual HRESULT STDMETHODCALLTYPE CreateSession(IUserActivitySession** result) = 0;
		};

		MIDL_INTERFACE("9DC40C62-08C4-47AC-AA9C-2BB2221C55FD")
		IUserActivity2 : public wclCommon::WinApi::IInspectable
		{
		public:
			virtual HRESULT STDMETHODCALLTYPE ToJson(wclCommon::WinApi::HSTRING* result) = 0;
		};

		MIDL_INTERFACE("E7697744-E1A2-5147-8E06-55F1EEEF271C")
		IUserActivity3 : public wclCommon::WinApi::IInspectable
		{
		public:
			virtual HRESULT STDMETHODCALLTYPE get_IsRoamable(BOOLEAN* value) = 0;
			virtual HRESULT STDMETHODCALLTYPE put_IsRoamable(BOOLEAN value) = 0;
		};

		MIDL_INTERFACE("34A5C8B5-86DD-4AEC-A491-6A4FAEA5D22E")
		IUserActivityAttribution : public wclCommon::WinApi::IInspectable
		{
		public:
			virtual HRESULT STDMETHODCALLTYPE get_IconUri(wclCommon::WinApi::IUriRuntimeClass** value) = 0;
			virtual HRESULT STDMETHODCALLTYPE put_IconUri(wclCommon::WinApi::IUriRuntimeClass* value) = 0;
			virtual HRESULT STDMETHODCALLTYPE get_AlternateText(wclCommon::WinApi::HSTRING* value) = 0;
			virtual HRESULT STDMETHODCALLTYPE put_AlternateText(wclCommon::WinApi::HSTRING value) = 0;
			virtual HRESULT STDMETHODCALLTYPE get_AddImageQuery(BOOLEAN* value) = 0;
			virtual HRESULT STDMETHODCALLTYPE put_AddImageQuery(BOOLEAN value) = 0;
		};

		MIDL_INTERFACE("E62BD252-C566-4F42-9974-916C4D76377E")
		IUserActivityAttributionFactory : public wclCommon::WinApi::IInspectable
		{
		public:
			virtual HRESULT STDMETHODCALLTYPE CreateWithUri(
				wclCommon::WinApi::IUriRuntimeClass* iconUri, IUserActivityAttribution** value) = 0;
		};

		MIDL_INTERFACE("BAC0F8B8-A0E4-483B-B948-9CBABD06070C")
		IUserActivityChannel : public wclCommon::WinApi::IInspectable
		{
		public:
			virtual HRESULT STDMETHODCALLTYPE GetOrCreateUserActivityAsync(
				wclCommon::WinApi::HSTRING activityId, IUserActivityAsyncOperation** operation) = 0;
			virtual HRESULT STDMETHODCALLTYPE DeleteActivityAsync(wclCommon::WinApi::HSTRING activityId,
				wclCommon::WinApi::IAsyncAction** operation) = 0;
			virtual HRESULT STDMETHODCALLTYPE DeleteAllActivitiesAsync(
				wclCommon::WinApi::IAsyncAction** operation) = 0;
		};

		MIDL_INTERFACE("1698E35B-EB7E-4EA0-BF17-A459E8BE706C")
		IUserActivityChannel2 : public wclCommon::WinApi::IInspectable
		{
		public:
			virtual HRESULT STDMETHODCALLTYPE GetRecentUserActivitiesAsync(
				INT32 maxUniqueActivities,
				IUserActivitySessionHistoryItemAsyncOperation** operation) = 0;
			virtual HRESULT STDMETHODCALLTYPE GetSessionHistoryItemsForUserActivityAsync(
				wclCommon::WinApi::HSTRING activityId, wclCommon::WinApi::DateTime startTime,
				IUserActivitySessionHistoryItemAsyncOperation** operation) = 0;
		};

		MIDL_INTERFACE("B399E5AD-137F-409D-822D-E1AF27CE08DC")
		IUserActivityContentInfo : public wclCommon::WinApi::IInspectable
		{
		public:
			virtual HRESULT STDMETHODCALLTYPE ToJson(wclCommon::WinApi::HSTRING* result) = 0;
		};

		MIDL_INTERFACE("7C385758-361D-4A67-8A3B-34CA2978F9A3")
		IUserActivityFactory : public wclCommon::WinApi::IInspectable
		{
		public:
			virtual HRESULT STDMETHODCALLTYPE CreateWithActivityId(
				wclCommon::WinApi::HSTRING activityId, IUserActivity** value) = 0;
		};

		MIDL_INTERFACE("A0EF6355-CF35-4FF0-8833-50CB4B72E06D")
		IUserActivityRequest : public wclCommon::WinApi::IInspectable
		{
		public:
			virtual HRESULT STDMETHODCALLTYPE SetUserActivity(IUserActivity* activity) = 0;
		};

		MIDL_INTERFACE("0C30BE4E-903D-48D6-82D4-4043ED57791B")
		IUserActivityRequestManager : public wclCommon::WinApi::IInspectable
		{
		public:
			virtual HRESULT STDMETHODCALLTYPE add_UserActivityRequested(
				IUserActivityRequestedEvent* handler,
				wclCommon::WinApi::EventRegistrationToken* token) = 0;
			virtual HRESULT STDMETHODCALLTYPE remove_UserActivityRequested(
				wclCommon::WinApi::EventRegistrationToken token) = 0;
		};

		MIDL_INTERFACE("AE434D78-24FA-44A3-AD48-6EDA61AA1924")
		IUserActivitySession : public wclCommon::WinApi::IInspectable
		{
		public:
			virtual HRESULT STDMETHODCALLTYPE get_ActivityId(wclCommon::WinApi::HSTRING** value) = 0;
		};

		MIDL_INTERFACE("E8D59BD3-3E5D-49FD-98D7-6DA97521E255")
		IUserActivitySessionHistoryItem : public wclCommon::WinApi::IInspectable
		{
		public:
			virtual HRESULT STDMETHODCALLTYPE get_UserActivity(IUserActivity** value) = 0;
			virtual HRESULT STDMETHODCALLTYPE get_StartTime(wclCommon::WinApi::DateTime* value) = 0;
			virtual HRESULT STDMETHODCALLTYPE get_EndTime(wclCommon::WinApi::IDateTimeReference** value) = 0;
		};

		MIDL_INTERFACE("94757513-262F-49EF-BBBF-9B75D2E85250")
		IUserActivityVisualElements : public wclCommon::WinApi::IInspectable
		{
		public:
			virtual HRESULT STDMETHODCALLTYPE get_DisplayText(wclCommon::WinApi::HSTRING* value) = 0;
			virtual HRESULT STDMETHODCALLTYPE put_DisplayText(wclCommon::WinApi::HSTRING value) = 0;
			virtual HRESULT STDMETHODCALLTYPE get_Description(wclCommon::WinApi::HSTRING* value) = 0;
			virtual HRESULT STDMETHODCALLTYPE put_Description(wclCommon::WinApi::HSTRING value) = 0;
			virtual HRESULT STDMETHODCALLTYPE get_BackgroundColor(wclCommon::WinApi::Color* value) = 0;
			virtual HRESULT STDMETHODCALLTYPE put_BackgroundColor(wclCommon::WinApi::Color value) = 0;
			virtual HRESULT STDMETHODCALLTYPE get_Attribution(IUserActivityAttribution** value) = 0;
			virtual HRESULT STDMETHODCALLTYPE put_Attribution(IUserActivityAttribution* value) = 0;
			virtual HRESULT STDMETHODCALLTYPE put_Content(wclCommon::WinApi::IAdaptiveCard* value) = 0;
			virtual HRESULT STDMETHODCALLTYPE get_Content(wclCommon::WinApi::IAdaptiveCard** value) = 0;
		};

		MIDL_INTERFACE("CAAE7FC7-3EEF-4359-825C-9D51B9220DE3")
		IUserActivityVisualElements2 : public wclCommon::WinApi::IInspectable
		{
		public:
			virtual HRESULT STDMETHODCALLTYPE get_AttributionDisplayText(
				wclCommon::WinApi::HSTRING* value) = 0;
			virtual HRESULT STDMETHODCALLTYPE put_AttributionDisplayText(
				wclCommon::WinApi::HSTRING value) = 0;
		};

		MIDL_INTERFACE("C8C005AB-198D-4D80-ABB2-C9775EC4A729")
		IUserActivityChannelStatics : public wclCommon::WinApi::IInspectable
		{
		public:
			virtual HRESULT STDMETHODCALLTYPE GetDefault(IUserActivityChannel** result) = 0;
		};

		MIDL_INTERFACE("8E87DE30-AA4F-4624-9AD0-D40F3BA0317C")
		IUserActivityChannelStatics2 : public wclCommon::WinApi::IInspectable
		{
		public:
			virtual HRESULT STDMETHODCALLTYPE DisableAutoSessionCreation() = 0;
			virtual HRESULT STDMETHODCALLTYPE TryGetForWebAccount(wclCommon::WinApi::IWebAccount* account,
				IUserActivityChannel** result) = 0;
		};

		MIDL_INTERFACE("9988C34B-0386-4BC9-968A-8200B004144F")
		IUserActivityContentInfoStatics : public wclCommon::WinApi::IInspectable
		{
		public:
			virtual HRESULT STDMETHODCALLTYPE FromJson(wclCommon::WinApi::HSTRING value,
				IUserActivityContentInfo** result) = 0;
		};

		MIDL_INTERFACE("C0392DF1-224A-432C-81E5-0C76B4C4CEFA")
		IUserActivityRequestManagerStatics : public wclCommon::WinApi::IInspectable
		{
		public:
			virtual HRESULT STDMETHODCALLTYPE GetForCurrentView(
				IUserActivityRequestManager** result) = 0;
		};

		MIDL_INTERFACE("8C8FD333-0E09-47F6-9AC7-95CF5C39367B")
		IUserActivityStatics : public wclCommon::WinApi::IInspectable
		{
		public:
			virtual HRESULT STDMETHODCALLTYPE TryParseFromJson(
				wclCommon::WinApi::HSTRING json, IUserActivity** result) = 0;
			virtual HRESULT STDMETHODCALLTYPE TryParseFromJsonArray(
				wclCommon::WinApi::HSTRING json, IUserActivityVector** result) = 0;
			virtual HRESULT STDMETHODCALLTYPE ToJsonArray(IUserActivityIterable* activities,
				wclCommon::WinApi::HSTRING* result) = 0;
		};

		MIDL_INTERFACE("35095983-7790-5974-a660-1c2dbdd2efa7")
		IUserActivityAsyncOperation : public wclCommon::WinApi::IInspectable
		{
		public:
			virtual HRESULT STDMETHODCALLTYPE put_Completed(
				IUserActivityAsyncOperationCompletedHandler* handler) = 0;
			virtual HRESULT STDMETHODCALLTYPE get_Completed(
				IUserActivityAsyncOperationCompletedHandler** handler) = 0;
			virtual HRESULT STDMETHODCALLTYPE GetResults(IUserActivity** results) = 0;
		};

		MIDL_INTERFACE("d0d0c30a-255c-5238-a7a2-aa905d383919")
		IUserActivitySessionHistoryItemAsyncOperation : public wclCommon::WinApi::IInspectable
		{
		public:
			virtual HRESULT STDMETHODCALLTYPE put_Completed(
				IUserActivitySessionHistoryItemAsyncOperationCompletedHandler* handler) = 0;
			virtual HRESULT STDMETHODCALLTYPE get_Completed(
				IUserActivitySessionHistoryItemAsyncOperationCompletedHandler** handler) = 0;
			virtual HRESULT STDMETHODCALLTYPE GetResults(
				IUserActivitySessionHistoryItemVector** results) = 0;
		};

		MIDL_INTERFACE("b71f6fda-21f5-5bcd-b83b-6c9eadff2410")
		IUserActivityRequestedEvent : public IUnknown
		{
		public:
			virtual HRESULT STDMETHODCALLTYPE Invoke(IUserActivityRequestManager* sender,
				IUserActivityRequestedEventArgs** e) = 0;
		};

		MIDL_INTERFACE("652507c7-0bc6-5d0b-82be-97ad2257b685")
		IUserActivityAsyncOperationCompletedHandler : public IUnknown
		{
		public:
			virtual HRESULT STDMETHODCALLTYPE Invoke(IUserActivityAsyncOperation* asyncInfo,
				wclCommon::WinApi::AsyncStatus status) = 0;
		};

		MIDL_INTERFACE("3df67ad8-7d8d-52bd-9892-0ff9bf93fc80")
		IUserActivitySessionHistoryItemAsyncOperationCompletedHandler : public IUnknown
		{
		public:
			virtual HRESULT STDMETHODCALLTYPE Invoke(
				IUserActivitySessionHistoryItemAsyncOperation* asyncInfo, wclCommon::WinApi::AsyncStatus status) = 0;
		};

		MIDL_INTERFACE("A4CC7A4C-8229-4CFD-A3BC-C61D318575A4")
		IUserActivityRequestedEventArgs : public wclCommon::WinApi::IInspectable
		{
		public:
			virtual HRESULT STDMETHODCALLTYPE get_Request(IUserActivityRequest** value) = 0;
			virtual HRESULT STDMETHODCALLTYPE GetDeferral(wclCommon::WinApi::IDeferral** value) = 0;
		};
	}
}